@@ -1,28 +1,31 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2020 Rumma & Ko Ltd
+# Copyright 2009-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-
+import datetime
 from io import StringIO
 from lxml import etree
 
 from django.conf import settings
 from django.db import models
-from django.utils.translation import gettext_lazy as _
+from django.utils.translation import gettext_lazy as _, gettext
 from django.utils import timezone
 
 from lino.api import dd, rt
 from lino import mixins
+from lino.core import constants
+from lino.core.gfks import gfk2lookup
 from lino.modlib.gfks.mixins import Controllable
 from lino_xl.lib.topics.models import AddInterestField
 from lino.modlib.users.mixins import My, UserAuthored
-from lino.modlib.publisher.mixins import Publishable
+from lino.modlib.comments.mixins import Commentable
+from lino.modlib.publisher.mixins import Publishable, PreviewPublication
 # from lino.modlib.printing.mixins import PrintableType, TypedPrintable
 from lino.mixins.periods import CombinedDateTime
-# from lino.core.requests import BaseRequest
+from lino.core.requests import BaseRequest
 from lino.modlib.memo.mixins import Previewable
 from lino.utils import join_elems
-from etgen.html import E
+from etgen.html import E, tostring
 
 from .roles import BlogsReader
 
@@ -63,7 +66,7 @@ class EntryTypes(dd.Table):
 
 
 class Entry(UserAuthored, Controllable, CombinedDateTime,
-            Previewable, Publishable):
+            Previewable, Publishable, Commentable):
 
     class Meta:
         app_label = 'blogs'
@@ -78,7 +81,14 @@ class Entry(UserAuthored, Controllable, CombinedDateTime,
     entry_type = dd.ForeignKey('blogs.EntryType', blank=True, null=True)
     language = dd.LanguageField()
 
+    publisher_location = "bl"
+    publisher_page_template = "blogs/entry_page.html"
+    publisher_item_template = "blogs/entry_item.html"
+    preview_publication = PreviewPublication(label=_("Static View"))
+
     def __str__(self):
+        if self.title:
+            return self.title
         if self.pub_date:
             return _("{} by {}").format(self.pub_date, self.user)
         return u'%s #%s' % (self._meta.verbose_name, self.pk)
@@ -92,11 +102,56 @@ class Entry(UserAuthored, Controllable, CombinedDateTime,
 
     add_interest = AddInterestField()
 
+    def get_detail_action(self, ar):
+        user = ar.get_user()
+        if user.user_type.has_required_roles([BlogsReader]) and \
+            user == self.user:
+            return super().get_detail_action(ar)
+        return rt.models.blogs.PublicEntries.detail_action
+
+    @dd.htmlbox()
+    def preview(self, ar):
+        if ar is None:
+            return
+
+        env = settings.SITE.plugins.jinja.renderer.jinja_env
+        template = env.get_template(self.publisher_page_template)
+
+        context = self.get_preview_context(ar)
+
+        return template.render(**context)
+
+    def get_preview_context(self, ar):
+        context = super().get_preview_context(ar)
+        FileUsage = rt.models.albums.FileUsage
+        primary_image = FileUsage.objects.filter(
+            **gfk2lookup(FileUsage.owner, self), primary_image=True)[0].file
+        context.update(pi=primary_image)
+
+        if ar.__class__ == BaseRequest:
+            return context
+
+        if ar.get_user() == self.user:
+            button = ('<span style="float: right;">' +
+                tostring(ar.row_action_button(self,
+                    rt.models.blogs.MyEntries.detail_action, label='edit',
+                    icon_name=None, CLASS='pi pi-pencil')) +
+                '</span>')
+            context.update(editable=True, edit_button=button)
+
+        return context
+
     @classmethod
     def get_dashboard_items(cls, user):
-        qs = cls.objects.filter(Q(pub_date__isnull=False)).order_by("-pub_date")
+        qs = cls.objects.filter(pub_date__isnull=False).order_by("-pub_date")
         return qs[:5]
 
+    @classmethod
+    def get_dashboard_objects(cls, ar):
+        # qs = cls.objects.filter(pub_date__isnull=False).order_by('-pub_date')
+        # return qs[:5]
+        return cls.get_dashboard_items(ar.get_user())
+
 
     # @classmethod
     # def latest_entries(cls, ar, max_num=10, **context):
@@ -137,22 +192,63 @@ class EntryDetail(dd.DetailLayout):
     main = """
     title entry_type:12 id
     # summary
-    pub_date pub_time user:10 language:10 owner add_interest
-    body:60 topics.InterestsByController:20 #TaggingsByEntry:20
+    pub_date pub_time user:10 language:10 owner add_interest"""
+
+    if dd.is_installed('albums'):
+        main += "\nalbums.UsagesByController"
+
+    main += "\nbody:60 topics.InterestsByController:20 comments.CommentsByRFC:20 #TaggingsByEntry:20"
+
+class ItemLayout(dd.DetailLayout):
+    main = "layout"
+
+    layout = "meta:30 body_short_preview:70"
+
+    meta = """
+    user
+    pub_date
     """
 
 
 class Entries(dd.Table):
-    required_roles = set([BlogsReader])  # also for anonymous
-    # required_roles = set()  # also for anonymous
+    required_roles = set([BlogsReader])
+
     model = 'blogs.Entry'
     column_names = "id pub_date user entry_type title * body"
     order_by = ["-id"]
+
+    hide_top_toolbar = False
+
+    display_mode = 'list'
+    disabled_display_mode = None # Something to play with (TODO)
+
     insert_layout = """
     title
     entry_type
     """
+
     detail_layout = EntryDetail()
+    item_layout = ItemLayout()
+
+    @classmethod
+    def collect_extra_actions(cls):
+        yield dd.WrappedAction(
+            rt.models.blogs.PublicEntries.detail_action,
+            label=_("Preview"),
+            icon_name=None)
+
+
+class PublicEntries(Entries):
+    required_roles = set()  # also for anonymous
+    hide_top_toolbar = True
+    detail_layout = """
+    preview
+    comments.CommentsByRFC
+    """
+
+    @classmethod
+    def collect_extra_actions(cls):
+        return []
 
 
 class MyEntries(My, Entries):
