@@ -9,6 +9,7 @@ from lino.utils import join_elems
 
 from django.db import models
 from django.db.models import Q
+from django.db.models.functions import Cast, Concat, Now
 from django.conf import settings
 from django.core.validators import MaxValueValidator
 from django.core.validators import MinValueValidator
@@ -19,10 +20,12 @@ from django.utils import timezone
 from lino import mixins
 from lino.api import dd, rt, _, pgettext
 
+from lino.core.site import html2text
 from lino.mixins import Referrable
 # from lino.mixins.periods import DateRangeObservable
 from lino.utils.quantities import Duration
 from lino.modlib.checkdata.choicelists import Checker
+from lino.modlib.notify.api import send_notification
 from lino.modlib.printing.mixins import TypedPrintable
 from lino.modlib.printing.mixins import Printable
 from lino.modlib.users.mixins import UserAuthored, Assignable
@@ -36,7 +39,7 @@ from lino_xl.lib.google.mixins import GoogleCalendarSynchronized
 from .choicelists import (
     DurationUnits, Recurrencies, Weekdays, AccessClasses, PlannerColumns, EventEvents, DisplayColors)
 
-from .choicelists import TaskStates, EntryStates, GuestStates
+from .choicelists import TaskStates, EntryStates, GuestStates, NotifyBeforeUnits
 from .actions import UpdateGuests
 
 from .mixins import Component,Colored
@@ -353,8 +356,7 @@ class RecurrentEvent(mixins.BabelNamed, RecurrenceSet, EventGenerator,
         return False
 
 dd.update_field(
-    RecurrentEvent, 'every_unit',
-    default=Recurrencies.as_callable('yearly'), blank=False, null=False)
+    RecurrentEvent, 'every_unit', default='yearly', blank=False, null=False)
 
 
 
@@ -418,11 +420,16 @@ class Event(
     state = EntryStates.field(default='suggested')
     all_day = ExtAllDayField(_("all day"))
 
+    notify_before = models.IntegerField(verbose_name=_("Notify before"), null=True, blank=True)
+    notify_unit = NotifyBeforeUnits.field(default='minutes')
+    notified = models.BooleanField(default=False)
+
     move_next = MoveEntryNext()
 
     show_conflicting = dd.ShowSlaveTable(ConflictingEvents)
     allow_merge_action = False
 
+
     @classmethod
     def setup_parameters(cls, params):
         super(Event, cls).setup_parameters(params)
@@ -517,9 +524,10 @@ class Event(
         else:
             return d
 
-    def get_diplay_color(self):
+    def get_display_color(self):
         if self.room:
             return self.room.display_color
+        return settings.SITE.site_config.default_color
 
     def __str__(self):
         if self.summary:
@@ -884,6 +892,7 @@ class Event(
 
     @dd.virtualfield(dd.DisplayField(_("Reminder")))
     def reminder(self, request):
+        # Required by extensible. Otherwise not used.
         return False
     # reminder.return_type = dd.DisplayField(_("Reminder"))
 
@@ -919,6 +928,20 @@ class Event(
             self.summary = self.owner.update_cal_summary(
                 self.event_type, self.auto_type)
 
+    @classmethod
+    def get_generators_for_plan(cls, plan, partner=None):
+        # pre-select all Event objects that potentially will generate
+        # an invoice.
+        qs = super(Event, cls).get_generators_for_plan(plan, partner)
+        max_date = plan.get_max_date()
+        if max_date:
+            qs = qs.filter(start_date__lte=max_date)
+        if plan.min_date:
+            qs = qs.filter(start_date__gte=plan.min_date)
+        # print("20210801 xl.cal.Event", qs.query)
+        return qs
+
+
     # def save(self, *args, **kwargs):
     #     if "Weekends" in str(self.owner):
     #         if not self.end_date:
@@ -928,6 +951,103 @@ class Event(
 dd.update_field(Event, 'user', verbose_name=_("Responsible user"))
 
 
+if dd.is_installed('notify'):
+    from lino.modlib.notify.choicelists import MessageTypes
+
+    MessageTypes.add_item('cal_event', _('Calendar Event'))
+
+    def get_notification_queryset():
+        Event = rt.models.cal.Event
+        db_engine = settings.DATABASES[Event.objects.db_manager().db]['ENGINE'].lower()
+
+        postgres, oracle = 'postgres' in db_engine, 'oracle' in db_engine
+
+        def get_duration():
+            if postgres:
+                return (models.When(
+                        notify_unit=NotifyBeforeUnits.get_by_name('minutes').value,
+                        then=Concat(
+                            models.F('notify_before'), models.Value(' minutes'),
+                            output_field=models.DurationField())),
+                    models.When(
+                        notify_unit=NotifyBeforeUnits.get_by_name('hours').value,
+                        then=Concat(
+                            models.F('notify_before'), models.Value(' hours'),
+                            output_field=models.DurationField())),
+                    models.When(
+                        notify_unit=NotifyBeforeUnits.get_by_name('days').value,
+                        then=Concat(
+                            models.F('notify_before'), models.Value(' days'),
+                            output_field=models.DurationField())),
+                    models.When(
+                        notify_unit=NotifyBeforeUnits.get_by_name('weeks').value,
+                        then=Concat(
+                            models.F('notify_before'), models.Value(' weeks'),
+                            output_field=models.DurationField())))
+            if oracle:
+                raise NotImplementedError(
+                    "Require implementation of int convertion to INTERVAL.\n" +
+                    "See: https://docs.djangoproject.com/en/3.2/ref/models/fields/#durationfield for help.")
+
+            return (models.When(notify_unit=NotifyBeforeUnits.get_by_name('minutes').value,
+                    then=models.F('notify_before') * 60 * 1000000),
+                models.When(notify_unit=NotifyBeforeUnits.get_by_name('hours').value,
+                    then=models.F('notify_before') * 60 * 1000000 * 60),
+                models.When(notify_unit=NotifyBeforeUnits.get_by_name('days').value,
+                    then=models.F('notify_before') * 60 * 1000000 * 60 * 24),
+                models.When(notify_unit=NotifyBeforeUnits.get_by_name('weeks').value,
+                    then=models.F('notify_before') * 60 * 1000000 * 60 * 24 * 7))
+
+        now = timezone.now()
+        tolerance = datetime.timedelta(minutes=scheduler_interval/2)
+        threshold = now + tolerance
+
+        return Event.objects.filter(notify_before__isnull=False,
+                notified=False, start_date__isnull=False
+            ).annotate(start_timed=models.Case( # dynamic start_time, db value if exists else CSH:00:00 # CSH -> calendar_start_hour
+                models.When(Q(start_time__isnull=True),
+                    then=models.Value(datetime.time(settings.SITE.calendar_start_hour, 0))),
+                default=models.F('start_time'),
+                output_field=models.TimeField()
+            )).annotate(start_datetime=Concat(
+                models.F('start_date'), models.Value(' '), models.F('start_timed'),
+                output_field=models.DateTimeField()
+            )).annotate(notify_before_duration=models.Case(
+                *get_duration(),
+                output_field=models.DurationField()
+            )).annotate(schedule=Cast(
+                models.F('start_datetime') - models.F('notify_before_duration'),
+                output_field=models.DateTimeField()
+            )).filter(schedule__lte=threshold)
+
+    def send_event_notifications(queryset):
+        message_type = MessageTypes.get_by_name('cal_event')
+        ar = rt.login(renderer=dd.plugins.memo.front_end.renderer, permalink_uris=True)
+        context = ar.get_printable_context()
+        template = rt.get_template('cal/Event/notification.eml')
+
+        for e in queryset:
+            def event_message(user, mail_mode):
+                if not hasattr(e, 'start_datetime'):
+                    e.start_datetime = datetime.datetime.combine(e.start_date, e.start_time)
+                subject = _("Upcoming Calendar Event @ {}").format(e.start_datetime.isoformat())
+                context.update(user=e.user, obj=e)
+                body = template.render(**context)
+                return (subject, body)
+
+            rt.models.notify.Message.emit_notification(
+                None, None, message_type, event_message, [(e.user, None)])
+
+            e.notified = True
+            e.save()
+
+    scheduler_interval = 5 # in minutes
+    @dd.schedule_often(every=scheduler_interval*60)
+    def event_notification_scheduler():
+        qs = get_notification_queryset()
+        send_event_notifications(qs)
+
+
 class EntryChecker(Checker):
     model = Event
     def get_responsible_user(self, obj):
@@ -1116,6 +1236,18 @@ class Guest(Printable):   # TODO: rename the model to "Presence"
             evt.save()
         return kw
 
+    @classmethod
+    def get_generators_for_plan(cls, plan, partner=None):
+        # pre-select all objects that potentially will generate
+        # an invoice.
+        qs = super(Guest, cls).get_generators_for_plan(plan, partner)
+        max_date = plan.get_max_date()
+        if max_date:
+            qs = qs.filter(event__start_date__lte=max_date)
+        if plan.min_date:
+            qs = qs.filter(event__start_date__gte=plan.min_date)
+        return qs
+
 
 def migrate_reminder(obj, reminder_date, reminder_text,
                      delay_value, delay_type, reminder_done):
@@ -1211,6 +1343,12 @@ dd.inject_field(
 events after the given date.""")
     ))
 
+dd.inject_field(
+    'system.SiteConfig',
+    'default_color',
+    DisplayColors.field("Color",default='Blue')
+)
+
 
 Reservation.show_today = ShowEntriesByDay('start_date')
 
@@ -1244,5 +1382,17 @@ if False:  # removed 20160610 because it is probably not used
     def pre_analyze(sender, **kw):
         sender.user_model.define_action(update_reminders=UpdateUserReminders())
 
+@dd.receiver(dd.post_startup)
+def setup_memo_commands(sender=None, **kwargs):
+    # See :doc:`/specs/memo`
+
+    if not sender.is_installed('memo'):
+        return
+
+    mp = sender.plugins.memo.parser
+
+    mp.register_django_model('cal_event', rt.models.cal.Event,
+        rnd=lambda obj: f"[cal_event {obj.id} {obj.summary}]",
+        title=lambda obj: obj.summary)
 
 from .ui import *
