@@ -30,7 +30,7 @@ class CountryCity(dd.Model):
         blank=True, null=True)
     zip_code = models.CharField(_("Zip code"), max_length=10, blank=True)
 
-    # active_fields = 'city zip_code'
+    active_fields = 'city zip_code'
     # active fields cannot be used in insert_layout
 
     @dd.chooser()
