@@ -1,5 +1,5 @@
 # Copyright 2008-2019 Rumma & Ko Ltd
-"""Estonian VAT declarations.  See :doc:`/specs/eevat`.
+"""Estonian VAT declarations.  See :doc:`/plugins/eevat`.
 
 """
 
@@ -15,5 +15,3 @@ class Plugin(ad.Plugin):
         m = m.add_menu("vat", site.plugins.vat.verbose_name)
         m.add_action('eevat.Declarations')
         m.add_action('eevat.DeclarationFields')
-
-
