@@ -1,18 +1,18 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2019 Rumma & Ko Ltd
+# Copyright 2019-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
 from lino.api import dd, rt, _
 
 
-def DEP(name, **kwargs):
+def area(name, **kwargs):
     kwargs = dd.str2kw('designation', name, **kwargs)
     # kwargs.update(designation=name)
     return rt.models.invoicing.Area(**kwargs)
 
 
 def objects():
-    yield DEP(_("First"))
-    yield DEP(_("Second"))
-    yield DEP(_("Third"))
-
+    yield area(_("First"))
+    if dd.plugins.invoicing.three_demo_areas:
+        yield area(_("Second"))
+        yield area(_("Third"))
