@@ -81,6 +81,9 @@ class Page(Hierarchical, Sequenced, Previewable, Publishable):
         for node in qs.order_by("seqno"):
             yield node
 
+    def __str__(self):
+        return self.title or self.ref or super().__str__()
+
     def get_absolute_url(self, **kwargs):
         if self.ref:
             if self.ref != 'index':
@@ -155,7 +158,7 @@ class PageDetail(dd.DetailLayout):
     main = """
     ref language parent seqno
     title
-    body pages.PagesByParent
+    body:70 pages.PagesByParent:30
     """
 
 
